<TeXmacs|0.3.4.3>

<style|article>

<\body>
  <assign|code|<macro|body|<surround|<vspace*|1fn>|<vspace|1fn><format|no
  indentation after>|<with|font family|tt|language|verbatim|<arg|body>>>>><ap\
  ply|title|The Mathemagix programming language><apply|author|Joris van der
  Hoeven><apply|address|Laboratoire de Mathmatiques (bt. 425)<format|next
  line>Universit Paris-Sud<format|next line>91405 Orsay Cedex<format|next
  line>France>

  <apply|maketitle>

  <\abstract>
    Mathemagix is intended to be the first free, type safe, compiled and
    general purpose computer algebra language. Mathemagix is very strongly
    typed, as it supports signatures or categories, which allow conditional
    exports, and interdependent tuples and functions. Yet, the user has a
    high control over the machine representation of data types and Mathemagix
    comes with a sophisticated object oriented type inheritance system.
  </abstract>

  <section|A short tutorial>

  <subsection|Functions>

  Functions can be declared either like

  <\code>
    square: Int-\<gtr\>Int == lambda (n:Int) do n*n;
  </code>

  or in the following more economic way:

  <\code>
    square (n:Int): Int == n*n;
  </code>

  Functions can both be used as arguments to other functions

  <\code>
    apply (f: Int-\<gtr\>Int, n: Int): Int == f(n);
  </code>

  and as return values:

  <\code>
    shift (by:Int) (n:Int) == n+by;
  </code>

  <subsection|Overloading>

  Mathemagix supports overloading of both functions and classes. For
  instance, <with|font family|tt|Array> may either take one type as an
  argument, or a type and a length. Similarly, the following would be
  permitted:

  <\code>
    x:Int \ \ \ := 3;

    x:String := "Hello";
  </code>

  In cases of ambiguity, an expression can be asserted to be disambiguised
  using the <with|font family|tt|:\<gtr\>> operator:

  <\code>
    print \<less\>\<less\> (x :\<gtr\> String) \<less\>\<less\> lf;
  </code>

  Mathemagix also supports implicit type convertion using the <with|font
  family|tt|convert> function. An implicit converter from <with|font
  family|tt|A> to <with|font family|tt|B> is defined using

  <\code>
    convert: A -\<gtr\> B == <with|font shape|italic|body>;
  </code>

  Converters may be specified to be <with|font family|tt|transitive>,
  <with|font family|tt|left transitive>, <with|font family|tt|right
  transitive> or <with|font family|tt|left right transitive>. When specifying
  a left transitive converter from <with|font family|tt|A> to <with|font
  family|tt|B> as well as a converter from <with|font family|tt|X> to
  <with|font family|tt|A>, the compiler automatically generates a converter
  from <with|font family|tt|X> to <with|font family|tt|B>. Similarly, a right
  transitive converter from <with|font family|tt|A> to <with|font
  family|tt|B> and a converter from <with|font family|tt|B> to <with|font
  family|tt|Y> yield a converter from <with|font family|tt|A> into <with|font
  family|tt|Y>. Finally, a transitive converter from <with|font family|tt|A>
  to <with|font family|tt|B> together with converters from <with|font
  family|tt|X> to <with|font family|tt|A> and <with|font family|tt|B> to
  <with|font family|tt|Y> yield converters from <with|font family|tt|X> to
  <with|font family|tt|B>, from <with|font family|tt|A> to <with|font
  family|tt|Y> and from <with|font family|tt|X> to <with|font family|tt|Y>.

  Sometimes, there is a natural way to transform an instance of type
  <with|font family|tt|A> into an instance of type <format|no line
  break><with|font family|tt|B>, although we do not want this transformation
  to be automatic. This may for instance be the cases when transforming an
  <with|font family|tt|Integer> into a <with|font family|tt|Double>. If such
  the case, you should write a function <with|font
  family|tt|cast:A-<format|no line break>\<gtr\>B> and type conversion always
  has to be performed manually.

  <subsection|Class definitions and importations>

  Routines can be grouped together into classes. The contents of a class may
  be imported by the main program or another class. Routines which have not
  explicitly been imported in a certain scope are unvisible:

  <\code>
    Test: $Class

    with {

    \ \ hello() == print \<less\>\<less\> "Hi there" \<less\>\<less\> lf;

    \ \ salut() == print \<less\>\<less\> "Salut" \<less\>\<less\> lf;

    }

    \;

    main () == {

    \ \ import Test; \ \ \ \ \ \ \ \ // makes hello visible

    \ \ hello(); \ \ \ \ \ \ \ \ \ \ \ \ // prints Hi there

    }
  </code>

  Since we did not use the routine <with|font family|tt|salut>, the following
  would also have been possible:

  <\code>
    main () == {

    \ \ import hello as hi from Test; \ // makes hello visible as hi, but not
    salut

    \ \ hi(); \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ // prints Hi
    there

    }
  </code>

  <subsection|Type inheritance>

  Classes are also used to define new data structures:

  <\code>
    Point: $Class

    with {

    \ \ intern x, y: Int;

    }

    \;

    main () == {

    \ \ import Point;

    \ \ P: Point := [ 2, 3 ];

    \ \ print \<less\>\<less\> P \<less\>\<less\> lf;

    }
  </code>

  Other classes can be inherited from the <with|font family|tt|Point> class:

  <\code>
    Colored_Point : $Point

    with {

    \ \ intern color: Int;

    }
  </code>

  Left and right transitive converters from <with|font family|tt|alias
  Colored_Point> to <with|font family|tt|alias Point> are automatically
  created by the compiler as a consequence of the above inheritance.

  <subsection|Separation of interfaces and implementations>

  The definition of any constant symbol may be postponed to a later stage,
  which allows the separation of interfaces in dedicated header files and the
  actual implementations. For instance,

  <\code>
    Test: $Class with { hello () -\<gtr\> (); }

    ...

    Test == add { hello(): () == print \<less\>\<less\> "Hi there"; }
  </code>

  Of course, it should be possible for the compiler to determine the type of
  the constant from its name and possible extra information. Therefore, one
  should not use overloading to heavily when using these constructions.

  Usually, the compiler should be able to determine the representation type
  of a class from its signature (i.e. from the interface). For private
  representation types, the definition can nevertheless be postponed to the
  implementation:

  <\code>
    T: $Class with {

    \ \ intern hidden;

    \ \ routine: () -\<gtr\> ();

    }

    \;

    T == add {

    \ \ intern <with|font family|tt|font shape|italic|data>;

    \ \ routine(): () == <with|font family|tt|font shape|italic|body>;

    }
  </code>

  The hidden part can also come after (but always after) a certain number of
  visible intern fields in the interface. Classes with a hidden part are
  necessarily indirect.

  It is also possible extend a class with additional routines <with|font
  shape|italic|as long as> its memory representation does not change (more
  precisely, only constant exports can be added to the class). For instance,
  the <with|font family|tt|Point> class from above may be extended using

  <\code>
    extend Point with {

    \ \ method translate_right (dx:Int) == { x += dx; }

    }
  </code>

  Again, it is possible to separate the interface and the actual
  implementation:

  <\code>
    extend Point with method translate_right: Int -\<gtr\> ();

    ...

    extend Point add method translate_right (dx:Int) == { x += dx; }
  </code>

  Possible extensions of a class should be made in the same block as the
  class definition itself.

  <subsection|Abstract classes>

  It is also possible to define abstract classes which cannot be
  instanciated:

  <\code>
    Ring: $Class with abstract {

    \ \ convert: Integer -\> %;

    \ \ +: (%,%) -\> %;

    \ \ -: (%,%) -\> %;

    \ \ *: (%,%) -\> %;

    }
  </code>

  The exports of such a class only get a meaning in concrete derived classes:

  <\code>
    Gaussian_Integer : $Ring

    with intern {

    \ \ re: Integer;

    \ \ im: Integer;

    }

    \;

    Gaussian_Integer ==

    add {

    \ \ convert (x:Integer) : % == [ x, 0 ];

    \ \ (x:%) + (y:%) : % == [ x.re + y.re, x.im + y.im];

    \ \ (x:%) - (y:%) : % == [ x.re - y.re, x.im - y.im];

    \ \ (x:%) * (y:%) : % == [ x.re*y.re - x.im*y.im, x.re*y.im + x.im*y.re
    ];

    }
  </code>

  The <with|font family|tt|%> symbol stands for ``the current class''. This
  should not be replaced by <with|font family|tt|Ring> in the definition of
  the abstract <with|font family|tt|Ring> class, since <with|font
  family|tt|%> may change during class inheritance.

  <subsection|Dependent types and classes as parameters>

  It is possible to define parameterized classes as follows:

  <\code>
    Polynomial (R:$Ring): $Ring

    with intern { coeffs:Array(R); }

    ...
  </code>

  Here <with|font family|tt|R:$Ring> means that <with|font family|tt|R>
  stands for a generic class derived from <with|font family|tt|Ring>. This is
  also the way Mathemagix passes classes as parameters. The parameters of a
  function or class definition may be interdependent, as in the following
  example:

  <\code>
    roots: (R:$Ring, Polynomial(R)) -\<gtr\> (E:$Algebra(Polynomial(R)),
    Array(E))
  </code>

  Furthermore, a type must always be constant when instanciated. For
  instance,

  <\code>
    i:Int := 5;

    a:Array(Int,i);

    i++;
  </code>

  and

  <\code>
    R:$Ring := Integer;

    import R;

    x:R := 5;

    R := Polynomial(R);
  </code>

  are incorrect, since the types of <with|font family|tt|a> resp. <with|font
  family|tt|x> would no longer be available after the changes of <with|font
  family|tt|i> resp. <format|no line break><with|font family|tt|R>. Types
  should also be <with|font shape|italic|persistant>, so that <with|font
  family|tt|Array(Int,random(10))> is not a valid type.

  <section|Control structures>

  <subsection|Blocks>

  The main control structure is a sequence of instructions, with optional
  declarations for local variables and local types:

  <\code>
    {

    \ \ <with|font shape|italic|decl>;

    \ \ ...<format|no page break after>

    \ \ <with|font shape|italic|decl>;<format|no page break after>

    <format|no page break after>

    \ \ <with|font shape|italic|inst>;<format|no page break after>

    \ \ ...<format|no page break after>

    \ \ <with|font shape|italic|inst>;

    <format|no page break before>}
  </code>

  The last instruction may also be a return value and blocks may be used as
  values. For instance,

  <\code>
    print \<less\>\<less\> { i:Int := 2; i+2; }
  </code>

  will print <with|font family|tt|4>. You may always exit the enclosing
  function of a block using the <with|font family|tt|return> keyword. The
  <with|font family|tt|return> keyword takes a parameter, if the enclosing
  function should return a value. Like in C++, it is also possible make new
  declarations after some instructions. In that case the compiler puts the
  missing braces of the corresponding subblock.

  <subsection|Conditions>

  The syntax of the <with|font family|tt|if-then-else> statement is similar
  to C:

  <\code>
    if <with|font shape|italic|cond> then <with|font shape|italic|inst>;
  </code>

  or

  <\code>
    if <with|font shape|italic|cond>

    then <with|font shape|italic|inst>;

    else <with|font shape|italic|inst>;
  </code>

  We may also use the <with|font family|tt|if-then-else> statement as an
  expression as follows:

  <\code>
    cout \<less\>\<less\> value if a=b then "yes" else { <with|font
    shape|italic|inst>; "no"; }
  </code>

  <subsection|Case distinction>

  A <with|font family|tt|case> distinction has the following syntax:

  <\code>
    case <with|font shape|italic|expr> in {

    \ \ <with|font shape|italic|match>, ..., <with|font shape|italic|match>
    do

    \ \ \ \ <with|font shape|italic|block>;

    \ \ ...

    \ \ <with|font shape|italic|match>, ..., <with|font shape|italic|match>
    do

    \ \ \ \ <with|font shape|italic|block>;

    \ \ else

    \ \ \ \ <with|font shape|italic|block>;

    }
  </code>

  The <with|font family|tt|else> part is optional. Inside the blocks, the
  <with|font family|tt|break> and <with|font family|tt|continue> keywords may
  respectively be used to exit the case statement and to jump to the next
  case.

  All possible matches for the expression must be expressions of the same
  type, which are tried sequentially. If all matches are constants, the
  compiler may decide to create a table or a hashtable for speed
  optimization. Fast lookup tables will be created for sure if the matches
  are more than 3 consecutive integers.

  <subsection|Loops>

  A loop is formed using the keywords <with|font family|tt|for>, <with|font
  family|tt|while>, <with|font family|tt|until>, <with|font family|tt|step>
  and <with|font family|tt|loop>:

  <\code>
    loop {

    \ \ <with|font shape|italic|forever>;

    }

    \;

    for i:=1 while i\<less\>=10 step i++ loop {<format|no page break after>

    \ \ <with|font shape|italic|block>;

    <format|no page break before>}
  </code>

  Inside a loop, the keywords <with|font family|tt|break> and <with|font
  family|tt|continue> may be used to exit the loop or to continue with the
  next step. Infinite loops may be used as values. In this case, the
  <with|font family|tt|break> keyword takes the return value of the loop as
  an argument.

  <subsection|Generators>

  It is also possible to create abstract control structures using the
  <with|font family|tt|generate> and <with|font family|tt|yield> keywords:
  For instance, the following piece of code has type <with|font
  family|tt|Generator(Int)>:

  <\code>
    gen: Generator(Int) := generate {

    \ \ print \<less\>\<less\> "Hallo" \<less\>\<less\> lf;

    \ \ yield 1;

    \ \ print \<less\>\<less\> "Hop" \<less\>\<less\> lf;

    \ \ yield 10;

    }
  </code>

  Generators are used in <with|font family|tt|for-in> loops. Using the above
  generator <with|font family|tt|gen>, the following piece of code

  <\code>
    i: Int;

    for i in gen loop print \<less\>\<less\> i \<less\>\<less\> lf;
  </code>

  will print

  <\code>
    Hallo

    1

    Hop

    10
  </code>

  The <with|font family|tt|for-in> construct can be used in combination with
  the <with|font family|tt|while>, <with|font family|tt|repeat> and
  <with|font family|tt|step> keywords. Several <with|font family|tt|for-in>,
  <with|font family|tt|><with|font family|tt|while>, <with|font
  family|tt|repeat> and <with|font family|tt|step> constructs can occur in
  the same loop:

  <\code>
    count := 0;

    for i in gen_1

    for j in gen_2

    while count \<less\> out_counted

    step count++

    loop {

    \ \ <with|font shape|italic|do_something>

    }

    print \<less\>\<less\> "The loop has cycled " \<less\>\<less\> count
    \<less\>\<less\> " times" \<less\>\<less\> lf;
  </code>

  A very standard iterator is

  <\code>
    for i in 1..10 loop print \<less\>\<less\> i \<less\>\<less\> lf;
  </code>

  <subsection|Exception handling>

  In order to handle exceptions, we use a <with|font
  family|tt|try-catch-raise> mechanism. Exceptions may simply be raised using
  a <with|font family|tt|raise> instruction:

  <\code>
    <with|font shape|italic|instrs>;<format|no page break after>

    if <with|font shape|italic|cond> then raise <with|font
    shape|italic|exception>;<format|no page break after>

    <with|font shape|italic|instrs>;
  </code>

  In this piece of code, <with|font family|tt|font shape|italic|exception>
  must be an expression of a non ambiguous type. A piece of code in which
  exceptions may occur, together with their corresponding exception handlers,
  are declared as follows:

  <\code>
    try {

    \ \ <with|font shape|italic|block>; // code in which exceptions of types
    <with|font shape|italic|type_1>, ..., <with|font shape|italic|type_n> may
    occur

    \;

    \ \ catch (<with|language|english|<with|font shape|italic|var>:<with|font
    shape|italic|type_1>>) == <with|font shape|italic|body>; \ // the
    exception handler for <with|font shape|italic|type_1>

    \ \ ...

    \ \ catch (<with|language|english|<with|font shape|italic|var>:<with|font
    shape|italic|type_n>>) == <with|font shape|italic|body>; \ // the
    exception handler for <with|font shape|italic|type_n>

    }

    <with|font shape|italic|instrs>; // execution is resumed here after
    normal execution or error handling
  </code>

  On normal execution, the exception handlers are ignored (in the executable
  code, exception handlers are put in dedicated locations). If an exception
  is raised, execution is resumed in the first exception handler in the block
  which is able to handle it the exception. If no dedicated exception handler
  was found in the block, the same exception is reraised in the next
  enclosing block or calling function.

  A global top-level exception handler is provided to handle all uncaught
  exceptions. This handler displays all information which is relevant to the
  exception. In order to be able to do this, all exception types must inherit
  from the class <with|font family|tt|Exception>, which comes with a short
  string description of the exception and a virtual default exception
  handler. Notice that the fact that any exception is in particular of type
  <with|font family|tt|Exception> enables the user to catch generic
  exceptions as well.

  <section|Primitive classes>

  In this section, we describe all classes which are available from scratch
  in Mathemagix, which means that you do not have to include any files in
  order to use them.

  <subsection|Function types>

  The only class constructor whis is really a language primitive in the sense
  that they require no explicit importation is <with|font
  family|tt|-\<gtr\>>. The general syntax is

  <\code>
    (<with|font shape|italic|Domain_1>,...,<with|font shape|italic|Domain_k>)
    -\<gtr\> (<with|font shape|italic|Image_1>,...,<with|font
    shape|italic|Image_l>)
  </code>

  The brackets may me omitted if <with|font family|tt|<with|font
  shape|italic|k>=1> resp. <with|font family|tt|<with|font
  shape|italic|l>=1>. The domain and image types may be interdependent, as in

  <\code>
    Jordan: (K:$Field, Matrix K) -\<gtr\> (L:$Extension_Field(K), Matrix L)
  </code>

  The domain and image types can also come with default arguments.

  <subsection|Arithmetic classes>

  The arithmetic classes can be subdivided into architecture dependent and
  architecture independent classes. The following classes are architecture
  independent:

  <\code>
    Integer

    Natural

    Int (<with|font shape|italic|nr_bits>)

    Nat (<with|font shape|italic|nr_bits>)

    Float (<with|font shape|italic|mantissa_bits>, <with|font
    shape|italic|exponent_bits>)
  </code>

  The <with|font family|tt|Float> class implements floating point numbers
  using the usual IEEE norm. The following architecture dependent classes are
  specializations of the generic above classes:

  <\code>
    Int, Short_Int, Long_Int

    Nat, Short_Nat, Long_Nat

    Float, Double
  </code>

  We always guarantee that <with|font family|tt|Int> and <with|font
  family|tt|Nat> have the same precision. Furthermore, <with|font
  family|tt|Long_Int> and <with|font family|tt|Long_Nat> are twice as precise
  as <with|font family|tt|Int> and <with|font family|tt|Nat>, which are again
  twice as precise as <with|font family|tt|Short_Int> and <with|font
  family|tt|Short_Nat>. Usually, <with|font family|tt|Int> and <with|font
  family|tt|Nat> have the length of a word. Similarly, <with|font
  family|tt|Float> and <with|font family|tt|Double> have the precisions which
  can be treated efficiently by the processor.

  In order to write architecture independent code, Mathemagix provides
  compilation options which allow you to set the precisions of <with|font
  family|tt|Int>, <with|font family|tt|Float> and <with|font
  family|tt|Double> yourself.

  <subsection|Other atomic classes>

  Further very basic atomic classes are the following:

  <\code>
    Bool

    Char

    String

    Enumerate (<with|font shape|italic|symbol>, ..., <with|font
    shape|italic|symbol>)
  </code>

  Characters are always unsigned and strings have the same representation as
  <with|font family|tt|Array(Char)>. In particular, they may contain null
  characters and they are not necessarily terminated by a null character.

  <subsection|Arrays>

  Arrays of undefinite and definite lengths are provided:

  <\code>
    Array (<with|font shape|italic|Type>)

    Array (<with|font shape|italic|Type>, <with|font shape|italic|length>)
  </code>

  The square brackets are used for array access and construction, as in

  <\code>
    a:Array(Int) := [1,1,2,3,5];

    print \<less\>\<less\> a[2] \<less\>\<less\> lf;
  </code>

  When compiling using the special <with|font shape|italic|data safety>
  option, an exception is raised when an attempt is made to access a field
  which is out of range.

  <subsection|Records>

  Record classes have the following syntax:

  <\code>
    Record (<with|font shape|italic|field>, ..., <with|font
    shape|italic|field>)
  </code>

  Each field is of one of the following forms:

  <\code>
    <with|font shape|italic|var>:<with|font shape|italic|Type>

    <with|font shape|italic|var>:<with|font shape|italic|Type> == <with|font
    shape|italic|default>
  </code>

  Data access is permitted using the <with|font family|tt|.> operator. Square
  brackets are used in order to create records, and you may omit those
  parameters which have a default initializer. Furthermore, you may specify
  fields by there name, like in the following example:

  <\code>
    T ==\<gtr\> Record (x:Int, y:Int, z:Int == 10);

    import T;

    a:T := [1,2]; \ \ \ \ \ \ \ \ \ \ // [1,2,10]

    b:T := [y==8, 7, 9]; \ \ \ // [7,8,9]

    c:T := [y==6, 5]; \ \ \ \ \ \ // [5,6,10]
  </code>

  It should be noticed that <with|font family|tt|Record> is more or less
  synonym for <with|font family|tt|with public intern>. Indeed, intern
  exports may also come with default initializers and a default constructor
  using <with|font family|tt|[]> is automatically generated by the compiler
  if all intern exports are public or locked.

  <subsection|Generators>

  Generators classes

  <\code>
    Generator(<with|font shape|italic|Type>)
  </code>

  have already been discussed before. By default, range generators <with|font
  family|tt|a..b> are provided as well as an implicit left transitive
  converter from <with|font family|tt|Array(<with|font shape|italic|Type>)>
  or <with|font family|tt|Array(<with|font shape|italic|Type>,<with|font
  shape|italic|length>)> to <with|font family|tt|Generator(<with|font
  shape|italic|Type>)>. The square brackets can be used to convert from
  <with|font family|tt|Generator(<with|font shape|italic|Type>)> to
  <with|font family|tt|Array(<with|font shape|italic|Type>)>. For example,
  <with|font family|tt|[1..4]> produces the array <with|font
  family|tt|[1,2,3,4]>.

  <subsection|Unions>

  A union class has the form

  <\code>
    Union (<with|font shape|italic|field>, ..., <with|font
    shape|italic|field>)
  </code>

  where the fields have a similar syntax as fields in records. However, at
  most one field can take a default initial value.

  The representation type of a union consists of an <with|font family|tt|Int>
  which indicates the field which holds the data and space corresponding to
  the maximal size of a field. When compiling with the data safety option, an
  exception is raised if an attempt is made to read from a field which does
  not contain the data. Individual fields are not available for writes, since
  references to fields of unions would not be typesafe. You rather have to
  reassign the entire union a value using <with|font family|tt|[]>. For
  instance:

  <\code>
    T ==\<gtr\> Union (i:Int, s:String);

    import T;

    a:T := [i==5];

    print \<less\>\<less\> a \<less\>\<less\> lf;

    a := [s=="Hallo"];

    if a holds s then print \<less\>\<less\> "a now holds a string"
    \<less\>\<less\> lf;
  </code>

  Here we used the <with|font family|tt|holds> keyword in order to determine
  whether the union contains a particular field.

  Mathemagix also provides the following union-like classes:

  <\code>
    Potential (<with|font shape|italic|Type>)
  </code>

  Such a class <with|font family|tt|Potential(T)> is the union of an indirect
  class <with|font family|tt|T> and the symbolic <with|font family|tt|nil>
  constant. This class inherits all structure from <with|font family|tt|T>
  and provides a <with|font family|tt|cast> function towards <with|font
  family|tt|T>. The <with|font family|tt|Potential> construct is for instance
  useful for the implementation of lists. When compiling using the data
  safety option, an exception is raised whenever you attempt to cast
  <with|font family|tt|nil> to <with|font family|tt|T>.

  <section|User defined classes>

  A class should be seen as a list of <with|font shape|italic|exports> with
  <with|font shape|italic|modifiers>. The modifiers control the behaviour of
  the exports w.r.t. memory layout, inheritance, inlining, data protection,
  and so on. Classes are always of the type <with|font family|tt|with
  <with|font shape|italic|exports>;> and their values are of the form
  <with|font family|tt|add <with|font shape|italic|implementation>;>.

  Parts of the signature to which the same modifier applies can be grouped
  together using <with|font family|tt|{}>. For instance, in order to define
  three <with|font family|tt|inline> routines, one may use

  <\code>
    inline {

    \ \ f: Int -\<gtr\> Int;

    \ \ g: (String, String) -\<gtr\> String;

    \ \ h: Polynomial(Int) -\<gtr\> ();

    }
  </code>

  instead of

  <\code>
    inline f: Int -\<gtr\> Int;

    inline g: (String, String) -\<gtr\> String;

    inline h: Polynomial(Int) -\<gtr\> ();
  </code>

  Furthermore, modifiers may alter the behaviour of <with|font
  family|tt|import> and <with|font family|tt|inherit>:

  <\code>
    outline import T; \ \ \ \ \ \ // disables inlining during the importation
    from T
  </code>

  <subsection|Intern and extern exports and methods>

  Three types of exports are distinguished in mathemagix:

  <\description>
    <expand|item*|Extern exports.>This is the way to export a routine which
    is relevant to the class. For instance, the <with|font family|tt|Ring>
    class may come with an extern addition function <with|font
    family|tt|+:(%,%)-\<gtr\>%>. By default, the exports of a class are
    extern.

    <expand|item*|Intern exports.>This is the way to define common data
    fields of all instances of a given signature. In other words, intern
    exports contain information about the representation type. For instance,
    one may want to decide that any type of signature <with|font
    family|tt|AbstractWidget> comes with fields <with|font
    family|tt|x1,y1,x2,y2:Int> which determine a bounding box for the widget.
    Although intern exports make the signature less abstract, it is a very
    handy way to share natural common data fields of types of a given
    signature.

    <expand|item*|Methods.>Methods are really extern exports, but they enjoy
    a similar syntax as intern exports. They can be seen as the analogue of
    member functions in C++.
  </description>

  <subsection|Variables, constants and aliases>

  Exports may either be declared to be <with|font family|tt|variable> or
  <with|font family|tt|constant>, depending on whether they are modifyable or
  not. By default, intern exports are variable, while extern exports are
  constant. Variable exports correspond to variables relative to a class and
  their precise semantics still has to be understood better.

  Constant intern exports correspond to methods of the class. Actually, there
  is a <with|font family|tt|method> keyword, which is equivalent to
  <with|font family|tt|constant intern>. Methods enjoy a ``data access-based
  syntax'':

  <\code>
    T: $Class

    with {

    \ \ intern n: Int;

    \ \ method increase(): () == {

    \ \ \ \ n := n+1;

    \ \ }

    }

    \;

    x:T := [1];

    x.increase();
  </code>

  Furthermore, the <with|font family|tt|x.> can be omitted inside the
  definitions of methods (and in <with|font family|tt|with <format|no line
  break>x <format|no line break>do <format|no line break>{...}>, if we decide
  to support such a construct).

  Exports can also be defined to be aliases or references, using the
  <with|font family|tt|alias> keyword. In other words, <with|font
  family|tt|alias T> is the analogue of <with|font family|tt|T&> in C++.
  Assignments take either a variable alias or a tuple of variable aliases as
  their left hand argument:

  <\code>
    x,y: Int := (2,1);

    (x,y) := (x+y, x-y);

    // x=3 and y=1 at this stage
  </code>

  <subsection|Type inheritance>

  Exports can be of three types with respect to type inheritance:

  <\description>
    <expand|item*|Concrete exports.>These are exports which are required to
    have an implementation in the base type and which can not be redefined in
    derived classes.

    <expand|item*|Evolutive exports.>These are exports which have an
    implementation in the base type, but which may be redefined in the
    derived class.

    <expand|item*|Abstract exports.>These are exports which have no
    implementation in the base class. They should be implemented in some
    derived class. A class which has at least one abstract export is called
    an <with|font shape|italic|abstract class>. No instances of abstract
    classes can be created.
  </description>

  The first way to specify the inheritance mode of exports is illustrated by

  <\code>
    Base: $Class with ...;

    Derived : $Base

    with ...;
  </code>

  An alternative, more precise way to achieve the same effect is the
  following

  <\code>
    Base: $Class with ...;

    Derived: $Class

    with {

    \ \ intern x: Base;

    \ \ inherit x;

    }
  </code>

  The second strategy gives the user total control over which exports of the
  base class should be inherited from which particular fields in the derived
  class. This is very useful in the case of multiple inheritance. Moreover,
  using aliases, the user may implement virtually any complex inheritance
  scheme, including joins. For instance, assume that we want to implement the
  following multiple inheritance scheme with a join:

  <\expand>
    equation*
  <|expand>
    <\with>
      math font series
    <|with>
      medium
    <|with>
      <matrix|||<with|mode|text|font family|tt|B>||||\<swarrow\>||\<searrow\>\
      ||<with|mode|text|font family|tt|X>||||<with|mode|text|font
      family|tt|Y>||\<searrow\>||\<swarrow\>||||<with|mode|text|font
      family|tt|D>|||5|5>
    </with>
  </expand>

  This may for instance be achieved as follows:

  <\code>
    B: $Class with ...;

    \;

    X: $Class with {

    \ \ intern B_in_X: alias B;

    \ \ private inherit B_in_X;

    \ \ ...

    }

    \;

    Y: $Class with {

    \ \ intern B_in_Y: alias B;

    \ \ private inherit B_in_Y;

    \ \ ...

    }

    \;

    D: $Class with {

    \ \ intern {

    \ \ \ \ B_in_D: B;

    \ \ \ \ X_in_D: X;

    \ \ \ \ Y_in_D: Y;

    \ \ }

    \ \ inherit B_in_D, X_in_D, Y_in_D;

    \ \ ...

    }
  </code>

  When constructing an instance of <with|font family|tt|X> resp. <with|font
  family|tt|Y> from <with|font family|tt|D>, one should take care of letting
  <with|font family|tt|B_in_X> resp. <with|font family|tt|B_in_Y> point to
  <with|font family|tt|B_in_D>. An alternative way to implement the above
  scheme would be to let <with|font family|tt|X> concretely inherit from
  <with|font family|tt|B> and <with|font family|tt|Y> only virtually.

  As with imports, inheritance may change the export modifiers. For instance,
  an abstract export in the base class may be made concrete in the derived
  class (but not the other way around).

  <subsection|Data protection>

  The four keywords <with|font family|tt|public, protected, locked> and
  <with|font family|tt|private> determine the way exports are protected
  during importations or through inheritance:

  <\description>
    <expand|item*|Public exports.>These exports are public both for
    importation and inheritance.

    <expand|item*|Protected exports.>These exports are private for
    importation, but public for inheritance.

    <expand|item*|Locked exports.>These exports are public for importation,
    but they cannot be inherited.

    <expand|item*|Private exports.>These exports are private both for
    importation and inheritance.
  </description>

  During importation or inheritance, exports may be made ``more private''.
  This means that public exports can be made protected, locked or private.
  Similarly, protected or locked exports can be made private. By default,
  exports are public in the class definition and necessarily private in the
  implementation.

  <subsection|Conditional exports><label|cond-exports>

  Exports may be preceded by conditions which may be verified at compile
  time:

  <\code>
    extend Polynomial (R:$Ring)

    with {

    \ \ if R has d then d: % -\> %;

    }
  </code>

  Specialization is an important application of conditional exports:

  <\code>
    extend Polynomial (R:$Ring)

    with {<format|no page break after>

    \ \ if R=Int then {<format|no page break after>

    \ \ \ \ import {+,-} from %;

    \ \ \ \ inline (p:%)*(q:%) : % == <with|font shape|italic|body>;

    \ \ }

    }
  </code>

  Here <with|font family|tt|import {+,-} from %> tells the compiler to take
  the <with|font family|tt|+> and <with|font family|tt|-> functions from the
  generic implementation of polynomials, and to specialize it to the
  situation when <with|font family|tt|R=Int> (using inlining from <with|font
  family|tt|Int>) and export the resulting function as being inline.

  Exports are allowed to be redefined in conditional exports, like in the
  above case of <with|font family|tt|Polynomial(Int)>. The innermost
  conditional redefinition always takes precedence over the others. If
  multiple conditions are met a routine can only be redefined at at most one
  innermost level. For instance, the following code is illegal:

  <\code>
    T(A:$X,B:$Y): $Class with {

    \ \ ...

    \ \ if A=U then f():() == <with|font shape|italic|code_1>;

    \ \ if B=V then f():() == <with|font shape|italic|code_2>;

    }

    \;

    import T(U,V);
  </code>

  <subsection|Low level flow control>

  An export which is declared to be <with|font family|tt|inline> will be
  inlined. For instance,

  <\code>
    T: with inline square (x:Integer): Integer == x*x;

    import T;

    print \<less\>\<less\> square(3);
  </code>

  will have the same effect as

  <\code>
    print \<less\>\<less\> 3*3;
  </code>

  One might eventually consider a more elaborate form of inlining, where
  certain routines may be inlined into other routines of a certain kind only,
  or where the inlining may depend on the optimization flags which are given
  to the compiler. One way to do this would be to allow the keyword
  <with|font family|tt|inline> to take symbolic parameters, like in

  <\code>
    inline[optimize,heavy] square: Integer-\<gtr\>Integer;
  </code>

  Now <with|font family|tt|square> will be inlined when using optimized
  compilation, or when importing using

  <\code>
    inline[heavy] import T;
  </code>

  An <with|font family|tt|inline> without arguments in a signature would then
  be equivalent to <with|font family|tt|inline[all]> and the <with|font
  family|tt|inline> prefix will inline everything which matches either its
  parameters or <with|font family|tt|all>.

  <subsection|Low level data representation control>

  Basically, there are two important ways to represent aggregate data
  structures such as records or arrays: via a pointer or directly as a block
  of memory. The use of pointers has the advantage of flexability, while the
  use of blocks of memory saves the cost of an indirection during data
  access, which may speed things up and avoid dynamic memory allocation. In
  order to let the user choose, intern exports may be declared to be
  <with|font family|tt|direct> using

  <\code>
    Point: $Class with direct intern {<format|no page break after>

    \ \ x,y,z:Integer;

    <format|no page break before>}
  </code>

  In order to be able to determine the size of a class, only record types and
  power types like <with|font family|tt|Array(T,n)>, where <with|font
  family|tt|n> is constant relative to place where <with|font
  family|tt|Array(T,n)> is used are allowed to be direct. For instance,

  <\code>
    intern {

    \ \ constant n:Integer;

    \ \ array: Array(Real,n);

    }
  </code>

  is valid, since <with|font family|tt|n> is constant in the type <with|font
  family|tt|Array(Real,n)>. However, this class can not be made direct, since
  it would not be possible to determine the size of such a class at compile
  time. On the other hand, a class like

  <\code>
    Real_Array(n:Integer): $Class with direct intern { array: Array (Real,
    n); }
  </code>

  would be permitted, since <with|font family|tt|n> must be specified in the
  name of the class. In particular, this implies that <with|font family|tt|n>
  must be constant.

  As usual, <with|font family|tt|direct> classes may be made <with|font
  family|tt|indirect> during inheritance, importation and the declaration of
  variables. For instance, in the function

  <\code>
    f():() == {

    \ \ direct a:Array(Int,10);

    \ \ ...

    }
  </code>

  the array <with|font family|tt|a> will be allocated directly on the stack.
  Similarly, in the code

  <\code>
    A: $Class with { intern x,y:Int; }

    B: $A with { z:Int; }
  </code>

  instances of <with|font family|tt|B> will be pointers to records with
  <with|font family|tt|x>, <with|font family|tt|y> and <with|font
  family|tt|z>, because <with|font family|tt|B> directly inherits from
  <with|font family|tt|A> by default. If indirect inheritance is used

  <\code>
    A: $Class with { intern x,y:Int; }

    B: indirect $A with { z:Int; }
  </code>

  instances of <with|font family|tt|B> are records whose first field is an
  instance of the indirect type <with|font family|tt|A> (i.e. a pointer to a
  record with <with|font family|tt|x> and <with|font family|tt|y>) and
  <with|font family|tt|z>.

  The keywords <with|font family|tt|packed> and <with|font
  family|tt|unpacked> may also be provided by Mathemagix.

  <subsection|Compiler hints>

  Additional modifiers may be provided in order to give additional hints to
  the compiler on how to optimize the code or to automatically check certain
  properties of the program, thereby facilitating the job of verifying its
  correctness. Some suggestions are the following:

  <\description>
    <expand|item*|Persistant.>The <with|font family|tt|persistant> modifier
    states that the result of a function is the same each time we pass the
    same parameters. For instance, this is the case for most mathematical
    functions, but not for a random number generator. Persistent functions
    might be given a Maple-like remember option.

    <expand|item*|Clean.>A clean function is a function without any side
    effects. Side effects may be changes of global variables, changes in
    reference arguments or input/output actions.

    <expand|item*|Commutative.>The <with|font family|tt|commutative> modifier
    states that a function is commutative, a property which may be useful
    during optimization. Similar modifiers might be <with|font
    family|tt|associative>, <with|font family|tt|transitive>, etc. One might
    also consider a <with|font family|tt|property> keyword with a list of
    parameters, like in <with|font family|tt|property[commutative,associative\
    ]>.

    <expand|item*|Safe.>To specify that no exceptions may be raised by the
    function, either directly or indirectly by one of its subfunctions. One
    may also consider more precise modifiers like <with|font
    family|tt|raises[<with|font shape|italic|list>]> or <with|font
    family|tt|catches[<with|font shape|italic|list>]>, where <with|font
    family|tt|font shape|italic|list> is a precise list of all exception
    classes which may be raised or catched.
  </description>

  <section|Programming in the large>

  <subsection|File types>

  Large scale programs written in mathemagix consist of the following types
  of files:

  <surround|<expand|item*|Project files.>||<\description>
    Mathemagix <with|font family|tt|.mmp> project files describe the
    dependencies between the several components of a large project. Project
    files are much like makefiles, but they contain more semantics. They may
    for instance prevent name clashes at link time, perform some version
    control, etc.

    <expand|item*|Source files.>The real Mathemagix programs. No include
    files or references to libraries have to be mentioned in the source: \ a
    Mathemagix program which uses more than one source file necessarily
    provides one or more project files.

    <expand|item*|Target files.>Mathemagix either produces <with|font
    family|tt|.h>, <with|font family|tt|.c>, <with|font family|tt|.o> files,
    static and dynamic libraries or executable files. These files can either
    be stored in the same directories as the source files, or in dedicated
    repositories. It is also possible to generate other types of files like
    documentation.

    <expand|item*|Auxiliary files.>For optimization purposes, the project
    manager might need to create several types of auxiliary files with header
    information, dependencies or version control.

    <expand|item*|Extern files.>The project files also describe the links to
    other files which may be relevant to a project, such as makefiles which
    can automatically generate source or data, configuration scripts, etc.
  </description>>

  Independent compilation is supported as follows. The object <with|font
  family|tt|.o> files correspond to individual source files. The project file
  is responsible for finding the context, analyzing the dependencies and to
  check date and version information. One might also consider resource
  sharing during the creation of distinct <with|font family|tt|.o> files,
  instead of relaunching the compiler several times like in a makefile.

  <subsection|Project files>

  <subsection|Comments and documentation>

  <subsection|Switchable debugging facilities and consistency checks>

  <subsection|Debugging>

  <subsection|Profiling and calibration>

  <section|Uncertainties>

  <subsection|The constant modifier in relation to side effects>

  We propose to assume arguments of functions to be constants inside the
  function body. For instance, the following piece of code would be
  incorrect:

  <\code>
    count_back (n:Int): () == {<format|no page break after>

    \ \ while n\<gtr\>0 step n-- loop print \<less\>\<less\> n
    \<less\>\<less\> lf;

    <format|no page break before>}
  </code>

  This convention prevents from unforeseen side effects:

  <\code>
    f (a: Array (Int)): () == { a[0] := 3; print \<less\>\<less\> a
    \<less\>\<less\> lf; }

    a:Array(Int) := [1,2,3];

    f(a);

    // a would now contain [3,2,3].
  </code>

  The point is that we do not want to copy the contents of <with|font
  family|tt|a> in general, when passing it as a parameter to a function. But
  if we do not copy the contents of <with|font family|tt|a>, then its fields
  may get corrupted when <with|font family|tt|a> is considered to be
  variable, even though the length of <with|font family|tt|a> (for instance)
  may not change. We argue that if the user should be able to partially
  modify <with|font family|tt|a> in a function, then he should as well be
  able to modify all of <with|font family|tt|a>, and this should be visible
  in the specification of the function. A correct implementation of the above
  piece of code would be the following:

  <\code>
    f (cst_a: Array (Int)): () == {

    \ \ a: Array (Int) := copy cst_a;

    \ \ a[0] := 3;

    \ \ print \<less\>\<less\> a \<less\>\<less\> lf;

    }

    a:Array(Int) := [1,2,3];

    f(a);
  </code>

  However, the far reaching consequences of our convention have to be studied
  in further detail. First of all, for certain classes like <with|font
  family|tt|Int>, \ a trivial implicit converter from <with|font
  family|tt|constant Int> to <with|font family|tt|variable Int> might be
  useful. It should also be noted that the result of a function should be
  <with|font family|tt|variable>, by default. For instance,

  <\code>
    import Vector(Int,3);

    u: Vector(Int,3) := [1,2,3];

    v: Vector(Int,3) := [3,5,9];

    w: Vector(Int,3) := u+v;
  </code>

  should be valid code, which would not be the case if the result of
  <with|font family|tt|u+v> were a constant. However, when writing
  constructors, it is sometimes necessary to let the return type be
  <with|font family|tt|constant>. This is well illustrated by the following
  example:

  <\code>
    Name : $Record (first:String, last:String)

    with {

    \ \ point (variable first:String, variable last:String): % == [first,
    last];

    \ \ point (first:String, last:String): constant % == [first, last];

    }
  </code>

  As a consequence, <with|font family|tt|constant> and <with|font
  family|tt|variable> modify the data type. However, one should not confuse
  <with|font family|tt|variable> with <with|font family|tt|alias>, because
  the <with|font family|tt|variable> and <with|font family|tt|constant>
  modifiers are about intended meaning and they do not affect representation
  types.

  Another consequence of this clean scheme of data protection is that it is
  often necessary to provide two versions for constructors and data access
  functions, depending on whether we are constructing resp. accessing a
  constant, a variable or an alias. We might consider the creation of
  <with|font family|tt|constructor> and <with|font family|tt|access>
  modifiers, which would automatically take care of this.

  <subsection|Conditional intern exports>

  A nice approach for creating more general union types would be to allow
  conditional intern exports. For instance, the following class would
  describe a tree whose inner nodes are labeled with non zero natural numbers
  and whose outer nodes are labeled with strings:

  <\code>
    Tree: $Class with intern {

    \ \ status: Nat;

    \ \ if status \ = 0 then label \ \ : String;

    \ \ if status != 0 then children: Tree;

    }
  </code>

  By convention, subsequent conditional intern exports which the compiler can
  detect to be mutually exclusive will be put into unions.

  <subsection|Exception handling>

  It is not yet clear to us whether all exceptions which may occur in a
  function (or which can be received by a function) should be declared. On
  the one hand, because of their <with|font shape|italic|exceptional> nature,
  we would not like to put this burden on the user. However, this may make
  the overall exceptional behaviour of large scale programs quite
  unpredictable; one may as well argue that a specification of a function
  should also include its behaviour under exceptional circumstances. On the
  other hand, how to foresee all possible future exceptions which might occur
  in an abstract data structure?

  From the implementation point of view, the absence of the declaration of
  exceptional behaviour raises another problem: how to check whether a given
  exception matches a given type. Some dynamic type information seems to be
  necessary here, which conflicts with the spirit of the language.
</body>

<\initial>
  <\collection>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|reduction page bottom margin|15mm>
    <associate|page type|a4>
    <associate|reduction page left margin|25mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|disc-cv|6.2>
    <associate|cond-exports|4.5>
    <associate|rem-spec|6.1>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate>
      toc
    <|associate>
      <vspace*|1fn><with|font series|<quote|bold>|1 A short
      tutorial><htab|5mm>?<vspace|0.5fn>

      1.1 Functions<htab|5mm>?

      1.2 Overloading<htab|5mm>?

      1.3 Class definitions and importations<htab|5mm>?

      1.4 Type inheritance<htab|5mm>?

      1.5 Separation of interfaces and implementations<htab|5mm>?

      1.6 Abstract classes<htab|5mm>?

      1.7 Dependent types and classes as parameters<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|2 Control
      structures><htab|5mm>?<vspace|0.5fn>

      2.1 Blocks<htab|5mm>?

      2.2 Conditions<htab|5mm>?

      2.3 Case distinction<htab|5mm>?

      2.4 Loops<htab|5mm>?

      2.5 Generators<htab|5mm>?

      2.6 Exception handling<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|3 Primitive
      classes><htab|5mm>?<vspace|0.5fn>

      3.1 Function types<htab|5mm>?

      3.2 Arithmetic classes<htab|5mm>?

      3.3 Other atomic classes<htab|5mm>?

      3.4 Arrays<htab|5mm>?

      3.5 Records<htab|5mm>?

      3.6 Generators<htab|5mm>?

      3.7 Unions<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|4 User defined
      classes><htab|5mm>?<vspace|0.5fn>

      4.1 Intern and extern exports and methods<htab|5mm>?

      4.2 Variables, constants and aliases<htab|5mm>?

      4.3 Type inheritance<htab|5mm>?

      4.4 Data protection<htab|5mm>?

      4.5 Conditional exports<htab|5mm>?

      4.6 Low level flow control<htab|5mm>?

      4.7 Low level data representation control<htab|5mm>?

      4.8 Compiler hints<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|5 Programming in the
      large><htab|5mm>?<vspace|0.5fn>

      5.1 File types<htab|5mm>?

      5.2 Project files<htab|5mm>?

      5.3 Comments and documentation<htab|5mm>?

      5.4 Switchable debugging facilities and consistency checks<htab|5mm>?

      5.5 Debugging<htab|5mm>?

      5.6 Profiling and calibration<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|6
      Uncertainties><htab|5mm>?<vspace|0.5fn>

      6.1 The constant modifier in relation to side effects<htab|5mm>?

      6.2 Conditional intern exports<htab|5mm>?

      6.3 Exception handling<htab|5mm>?
    </associate>
  </collection>
</auxiliary>
