
/******************************************************************************
* MODULE     : mmx_extend.cc
* DESCRIPTION: generate generic extension code for C++ template libraries
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extend.hh>
#include <glue.hh>
#include <file.hh>
using std::fstream;
using std::ios;
using namespace mmx;

static bool verbose= true;
static bool so_flag= true;
static bool glue_flag= true;
static string output_dir= ".";
static string next_output= "";
static string local_lib_path;
static string user_lib_path;

string
get_lib_name (string root) {
  if (next_output != "") return next_output;
  if (output_dir == ".") return "mmx-" * root * ".so";
  int i, n= N(root);
  for (i=n-1; i>=0; i--)
    if (root[i] == '/') break;
  return output_dir * "/mmx-" * root (i+1, n) * ".so";
}

void
produce_extension (string arg) {
  string root= arg;
  if (ends (root, ".mmh")) root= root (0, N(root) - 4);
  string in_name  = root * ".mmh";
  string out_name = root * ".cpp";
  string lib_name = get_lib_name (root);

  string s;
  insert_source (arg, "cmdline");
  if (load (arg, s)) {
    cerr << "Couldn't open file " << in_name << "\n";
    exit (1);
  }
  list errors;
  generic prog= parse (s, errors);
  if (!nil (errors)) {
    print_errors (errors);
    exit (1);
  }
  if (true) {
    // ostream& fout= cout; (void) out_name;
    fstream fout (as_charp (out_name), ios::out);
    if (verbose) cout << "mmx-extend: producing " << symbol (out_name) << lf;
    if (extension_code (fout, root, prog, glue_flag)) exit (1);
  }
  if (so_flag) {
    string cpp = "g++ -fPIC -shared -I" * sys_include_path;
    string cmd = cpp * " " * out_name * " -o " * lib_name;
    if (verbose) cout << "mmx-extend: " << symbol (cmd) << lf;
    if (system (cmd)) exit (1);
    cmd = "rm -f " * out_name;
    if (verbose) cout << "mmx-extend: " << symbol (cmd) << lf;
    if (system (cmd)) exit (1);
  }
  next_output= "";
  so_flag= true;
  glue_flag= true;
}

void
process_option (char** args, int n, int& i) {
  string option = args[i];
  if (starts (option, "--")) option= option (1, N(option));
  if ((option == "-h") || (option == "-help")) {
    cout << "Options for mmx-extend:\n";
    cout << "  -c      : produce cpp file only for next argument\n";
    cout << "  -d dir  : specify output directory\n";
    cout << "  -g      : produce generic code only and not the glue\n";
    cout << "  -h      : display this help message\n";
    cout << "  -l      : equivalent to -d " << local_lib_path << "\n";
    cout << "  -s      : silent execution\n";
    cout << "  -o file : specify next output file\n";
    cout << "  -u      : equivalent to -d " << user_lib_path << "\n";
    cout << "  -v      : display the version\n";
    cout << "Please report bugs to <bugs@mathemagix.org>\n";
    exit (0);
  }
  if ((option == "-s") || (option == "-silent"))
    verbose= false;
  if ((option == "-o") || (option == "-output"))
    if (i+1 < n) next_output= args[++i];
  if ((option == "-d") || (option == "-dir"))
    if (i+1 < n) output_dir= args[++i];
  if ((option == "-l") || (option == "-local-libdir"))
    output_dir= local_lib_path;
  if ((option == "-u") || (option == "-user-libdir"))
    output_dir= user_lib_path;
  if ((option == "-v") || (option == "-version")) {
    cout << VERSION << "\n"; exit (0); }
  if ((option == "-c") || (option == "-cpp-code-only"))
    so_flag= false;
  if ((option == "-g") || (option == "-generic-code-only")) {
    so_flag= false; glue_flag= false; }
  if (option == "-test") {
    cout << "MMXDATA   = " << MMXDATA << "\n";
    cout << "MMXINCLUDE= " << MMXINCLUDE << "\n";
    cout << "MMXLIB    = " << MMXLIB << "\n";
    cout << "MMXSRC    = " << MMXSRC << "\n";
    exit (0);
  }
}

int
main (int argc, char** argv) {
  init_system ();
  local_lib_path= sys_lib_path;
  user_lib_path= sys_home * "/.mathemagix/lib";

  if (argc < 2) { 
    cout << "Usage: " << argv[0] << " --help\n";
    cout << "       " << argv[0] << " [options] files\n";
    return 0;
  }

  int i;
  for (i=1; i<argc; i++) {
    string arg = argv[i];
    if (starts (arg, "-")) process_option (argv, argc, i);
    else produce_extension (arg);
  }

  return 0;
}

